import { select, json, hierarchy } from 'd3';
import { dropdownMenu } from './dropdownMenu';

const svg = select('svg').attr('transform', `translate(10, 0)`);
const width = document.body.clientWidth;
const height = document.body.clientHeight;
svg
  .attr('width', width)
  .attr('height', height)


// State and Handlers
let data;
let paths;
let selectedPath;
let selectedSequence = [];
let selectedSequenceId = -1;
let selectedSequenceIndex = -1;
const sequenceG = svg.append('g').attr('transform', 'translate(50, 50)');

const updateSequence = (descendants, index) => {
  selectedSequence = Array();
  descendants.forEach((d, i) => selectedSequence.push(d.data.data));
  
  selectedSequenceId = selectedSequence[0].id;
  selectedSequenceIndex = index;
  render();
}

const onPathClicked = (path) => {
  selectedPath = path;
}


const render = () => {
  
  const length = 20;
  const margin = {right: 30, bottom: 50};
  
  if(selectedPath !== 'all') {
    data = data.filter(d => d.data.path === selectedPath)
  }
  data.forEach((window, index) => {
    
    const root = hierarchy(window);
    const descendants = root.descendants();
    const nodeEnter = sequenceG.selectAll(`.sequence-${index}`).data(descendants);
    nodeEnter
      .enter().append('rect')
        .attr('class', `sequence-${index}`)
        .attr('stroke', 'black')
        .attr('fill', 'black')
        .attr('width', length)
        .attr('height', length)
        .attr('y', margin.bottom * index)
        .attr('x', (d, i) => (margin.right + length) * i)
        .on('click', () => updateSequence(descendants, index));
    
  });
  
  // draw rectangle around selected sequences
  if(selectedSequenceId >= 0) {
    const selectEnter = sequenceG.selectAll(`.sequence-selection`).data([selectedSequenceId], d => d);
    selectEnter
      .enter().append('rect')
        .attr('class', 'sequence-selection')
        .attr('stroke', 'black')
        .attr('fill', 'yellow')
        .attr('width', (length + margin.right) * (selectedSequence.length))
        .attr('height', length + (margin.bottom/3))
        .attr('y', (margin.bottom * selectedSequenceIndex) - (margin.bottom/6))
        .attr('x', -margin.right/2)
      .merge(selectEnter)
        .style('opacity', d => (d === selectedSequenceId) ? 0.3 : 0)
      selectEnter.exit().remove();
  }
  
  const crumbs = sequenceG.selectAll('text').data(selectedSequence, d => String(d.id));
  crumbs
    .enter().append('text')
      .attr('class', 'breadcrumbs')
      .attr('fill', 'black')
      .attr('font-size', '20px')
      .attr('y', (d, i) => (data.length * margin.bottom) + (20 * i) + 40)
      .text(d => d.id)
  crumbs.exit().remove();
  
  select('#path-menu')
    .call(dropdownMenu, {
      options: paths,
      onOptionClicked: onPathClicked,
      selectedOption: selectedPath
    });
}


// Data Loading
json('data.json')
  .then(loadedData => {
    data = loadedData;
  	paths = data
      .map(d => d.data.path)
      .filter((value, index, self) => self.indexOf(value) === index)
    paths.push('all');
    
  selectedPath = 'all';
  render();
});